-- GLOBALS
--myappwidth = application:getDeviceWidth()
--myappheight = application:getDeviceHeight()
myappwidth = application:getContentWidth()
myappheight = application:getContentHeight()

function math.dist(x1, y1, x2, y2) return ((x2-x1)^2+(y2-y1)^2)^0.5 end
function math.angle(x1, y1, x2, y2) return math.atan2(y2 - y1, x2 - x1) end

function isColliding(a1, a2) -- two bitmap objects
	if(a1 == a2) then return false end
	local dx = a1.posx - a2.posx
	local dy = a1.posy - a2.posy
	if math.abs(dx) < a1.bmp:getWidth() / 2 + a2.bmp:getWidth() / 2 then
		if math.abs(dy) < a1.bmp:getHeight() / 2 + a2.bmp:getHeight() / 2 then
			return true
		end
	end
	
	return false
end

function isColliding2(a1, a2) -- two sprite objects
	if(a1 == a2) then return false end
	local dx = a1.posx - a2.posx
	local dy = a1.posy - a2.posy
	if math.abs(dx) < a1.w / 2 + a2.w / 2 then
		if math.abs(dy) < a1.h / 2 + a2.h / 2 then
			return true
		end
	end
	
	return false
end
-- GLOBALS

-- ******************************************************************************

-- ******************************************************************************

function loadCSV(file)

	local function csvsplit(line)
		local t, w, l2 = {}, nil, nil

		while #line > 0 do
			w, l2 = line:match("\"([^\"]*)\"[,\r\n]?(.*)") -- Check for quoted string
			if not w then w, line = line:match("([^,]*)[,\r\n]?(.*)") -- Non quoted 
			else line = l2 end
			if not w then break end --Nothing or issue, break

			t[#t + 1] = w
		end

		return t
	end

	local headers, csv = nil, {}

	for line in io.lines(file) do
		f = csvsplit(line)
		if not headers then -- Assume first line is headers
			headers = {} for n, v in ipairs(f) do headers[v] = n end
		else
			csv[#csv + 1] = f
		end
	end

	csv.getField = function(self, row, field) return self[row][headers[field]] end

	return csv

end
--csv=loadCSV("135-bis-bt.csv")
--print(csv:getField(4,"Value"))
--print(csv:getField(7,"Time")
 
-- ******************************************************************************

-- ******************************************************************************

require "json"

function saveData(filepath, value)
	local contents = json.encode(value)
	local file = io.open("|D|"..filepath, "w") -- create file
	file:write(contents) -- save json string in file
	io.close(file)
end

function getData(filepath)
	local value
	local file = io.open("|D|"..filepath, "r")
	if file then
		local contents = file:read("*a") -- read contents
		value = json.decode(contents) -- decode json
		io.close(file)
	end

	return value
end

--[[
--try to read information
local someData = getData("someData")

-- if no information, create it
if not someData then
	someData = {"some text", 42}
	saveData("someData.txt", someData)
	print("Creating someData")
else
	print("Read someData", someData[1], someData[2])
end
]]

-- ******************************************************************************

-- ******************************************************************************

--[[
A generic button class

This code is MIT licensed, see http://www.opensource.org/licenses/mit-license.php
(C) 2010 - 2011 Gideros Mobile 
]]

Button = Core.class(Sprite)

function Button:init(upState, downState) -- bitmaps
	self.upState = upState
	self.downState = downState

	self.focus = false

	self:updateVisualState(false)

	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
	self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)

	self:addEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
	self:addEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
	self:addEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
	self:addEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)
end

function Button:onMouseDown(event)
	if self:hitTestPoint(event.x, event.y) then
		self.focus = true
		self:updateVisualState(true)
		event:stopPropagation()
	end
end

function Button:onMouseMove(event)
	if self.focus then
		if not self:hitTestPoint(event.x, event.y) then
			self.focus = false;
			self:updateVisualState(false)
		end
		event:stopPropagation()
	end
end

function Button:onMouseUp(event)
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		self:dispatchEvent(Event.new("click"))
		event:stopPropagation()
	end
end

-- if button is on focus, stop propagation of touch events
function Button:onTouchesBegin(event)
	if self.focus then
		event:stopPropagation()
	end
end

-- if button is on focus, stop propagation of touch events
function Button:onTouchesMove(event)
	if self.focus then
		event:stopPropagation()
	end
end

-- if button is on focus, stop propagation of touch events
function Button:onTouchesEnd(event)
	if self.focus then
		event:stopPropagation()
	end
end

-- if touches are cancelled, reset the state of the button
function Button:onTouchesCancel(event)
	if self.focus then
		self.focus = false
		self:updateVisualState(false)
		event:stopPropagation()
	end
end

-- if state is true show downState else show upState
function Button:updateVisualState(state)
	if state then
		if self:contains(self.upState) then
			self:removeChild(self.upState)
		end
		
		if not self:contains(self.downState) then
			self:addChild(self.downState)
		end
	else
		if self:contains(self.downState) then
			self:removeChild(self.downState)
		end
		
		if not self:contains(self.upState) then
			self:addChild(self.upState)
		end
	end
end

-- ******************************************************************************

-- ******************************************************************************

ButtonText = Core.class(Sprite)

function ButtonText:init(text, tfont, tcolor, tscalex, tscaley, upState, downState)
	if text ~= nil then
		self.text = TextField.new(tfont, text, text) -- string
		self.text:setAnchorPoint(0.5, 0.5)
		self.text:setScale(tscalex, tscaley)
		self.text:setTextColor(tcolor or colorred01)
		self.textwidth = self.text:getWidth()
		self.textheight = self.text:getHeight()
		self:addChild(self.text)
		self.hastext = true
	else
		self.hastext = false
	end

	if upState ~= nil and downState ~= nil then
		self.upState = Bitmap.new(Texture.new(upState))-- png path, ...
		self.downState = Bitmap.new(Texture.new(downState)) -- png path, ...
		self.upState:setAnchorPoint(0.5, 0.5)
		self.downState:setAnchorPoint(0.5, 0.5)
		self.bmpwidth = self.upState:getWidth()
		self.bmpheight = self.upState:getHeight()
		self.hasbmp = true
	else
		self.hasbmp = false
	end

	if self.hasbmp then
		self:updateVisualState(false)
	end

	self.focus = false

	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
	self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)

	self:addEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
	self:addEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
	self:addEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
	self:addEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)
end

-- BUTTON FUNCTIONS
function ButtonText:setTextView(pcolor, pscalex, pscaley)
	self.text:setTextColor(pcolor)
	self.text:setScale(pscalex, pscaley)
end

function ButtonText:setText(ptext, xupper)
	if xupper then
		self.text:setText(string.upper(ptext))
	else
		self.text:setText(ptext)
	end
end

function ButtonText:setTextColor(xcolor)
	self.text:setTextColor(xcolor or 0xff0000)
end

function ButtonText:myShow()
	self:addChild(self.text)
	if self.downState ~= nil then self:addChild(self.downState) end
	if self.upState ~= nil then self:addChild(self.upState) end
--	if self:contains(self.downState) then self:addChild(self.downState) end
--	if self:contains(self.upState) then self:addChild(self.upState) end
end

function ButtonText:myHide()
	self:removeChild(self.text)
--	if self.downState ~= nil then self:removeChild(self.downState) end
--	if self.upState ~= nil then self:removeChild(self.upState) end
	if self:contains(self.downState) then self:removeChild(self.downState) end
	if self:contains(self.upState) then self:removeChild(self.upState) end
end

function ButtonText:updateVisualState(state)
	if self.hasbmp then
		-- if state is true show downState bitmap else show upState bitmap
		if state then
			if self:contains(self.upState) then
				self:removeChild(self.upState)
			end
			
			if not self:contains(self.downState) then
				self:addChild(self.downState)
			end

			if self:contains(self.text) then
				self:addChild(self.text)
			end
		else
			if self:contains(self.downState) then
				self:removeChild(self.downState)
			end
			
			if not self:contains(self.upState) then
				self:addChild(self.upState)
			end

			if self:contains(self.text) then
				self:addChild(self.text)
			end
		end
	end
end

-- BUTTON LISTENERS
function ButtonText:onMouseDown(event)
	if self:hitTestPoint(event.x, event.y) then
		self.focus = true
		self:updateVisualState(true)
		event:stopPropagation()
	end
end

function ButtonText:onMouseMove(event)
	if self.focus then
		if not self:hitTestPoint(event.x, event.y) then
			self.focus = false;
			self:updateVisualState(false)
		end
		event:stopPropagation()
	end
end

function ButtonText:onMouseUp(event)
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		self:dispatchEvent(Event.new("click"))
		event:stopPropagation()
	end
end

function ButtonText:onTouchesBegin(event)
	-- if button is on focus, stop propagation of touch events
	if self.focus then
		event:stopPropagation()
	end
end

function ButtonText:onTouchesMove(event)
	-- if button is on focus, stop propagation of touch events
	if self.focus then
		event:stopPropagation()
	end
end

function ButtonText:onTouchesEnd(event)
	-- if button is on focus, stop propagation of touch events
	if self.focus then
		event:stopPropagation()
	end
end

function ButtonText:onTouchesCancel(event)
	-- if touches are cancelled, reset the state of the button
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		event:stopPropagation()
	end
end

-- ******************************************************************************

-- ******************************************************************************

xCircle = Core.class(Sprite)

function xCircle:init(r, steps, color)
	color = color or 0xff0000

	local sin, cos, d2r = math.sin, math.cos, math.pi / 180

	local c = Shape.new()
	c:setFillStyle(Shape.SOLID, color)
	c:setLineStyle(0)
	c:beginPath()
	c:moveTo(r * sin(0 * d2r), r * cos(0 * d2r))
	for i = 0, 360, 360 / steps  do
	  c:lineTo(r * sin(i * d2r), r * cos(i * d2r))
	end    
	c:endPath()

	self:addChild(c)
end

-- ******************************************************************************

-- ******************************************************************************

CircularProgress = Core.class(Sprite)
 
function CircularProgress:init(radius, width, color, font, fontscale)
	self.radiusMin, self.radiusMax = radius - width / 2, radius + width / 2
	self.color = color
	self.font = font
	self:addChild(Path2D.new())
	local text = TextField.new(font, "-")
	text:setTextColor(color)
	text:setScale(fontscale or 1)
	self:addChild(text)
--	self:setValue(0.4)
end

function CircularProgress:setValue(val, xcolor, xalpha)
	local angs, ange = 0, math.pi * val * 1.999
	local rl, rh = self.radiusMin, self.radiusMax
	local ta = self:getChildAt(1)
	local ra = math.floor(val * 2)
--	ta:setLineColor(self.color, xalpha)
--	ta:setFillColor(self.color, xalpha)
	ta:setLineColor(xcolor, xalpha)
	ta:setFillColor(xcolor, xalpha)
	ta:setPath("MALAZ",
		math.sin(angs) * rl, -math.cos(angs) * rl,
		rl, rl, 0, ra, 1,
		math.sin(ange) * rl, -math.cos(ange) * rl,
		math.sin(ange) * rh, -math.cos(ange) * rh,
		rh, rh, 0, ra, 0,
		math.sin(angs) * rh, -math.cos(angs) * rh
		)
	ta:setLineThickness(4, 0.5)
	ta = self:getChildAt(2)
	ta:setText(("%d%%"):format(math.floor(val * 100)))
	ta:setTextColor(xcolor)
	ta:setAlpha(xalpha)
	local tax, tay, taw, tah = ta:getBounds(ta)
	ta:setAnchorPosition(tax + taw / 2, tay + tah / 2)

	return ta

end
 
--[[
application:setBackgroundColor(0)
local meter=CircularProgress.new(80,20,0xFF0000,nil,4)
meter:setPosition(100,100)
stage:addChild(meter)
 
local animSpeed=240
stage:addEventListener(Event.ENTER_FRAME,function ()
	local frame=Core.frameStatistics().frameCounter
	local frameIndex=frame%animSpeed
	local ratio=math.abs(frameIndex-animSpeed/2)/(animSpeed/2)
	meter:setValue(ratio)
end)
]]

-- ******************************************************************************

-- ******************************************************************************

xCenteredTextField = Core.class(Sprite)

function xCenteredTextField:init(ptext, pfont, pcolor, pscalex, pscaley, pposx, pposy)
	self.anchorpointx = 0.5
	self.anchorpointy = 0.5
	self.mytextfield = TextField.new(pfont, ptext, ptext)
	self.mytextfield:setTextColor(pcolor or 0xff0000)
	self.mytextfield:setScale(pscalex, pscaley)
	self.mytextfield:setAnchorPoint(self.anchorpointx, self.anchorpointy)
	self.mytextfield:setPosition(pposx, pposy)
	self:addChild(self.mytextfield)
end

function xCenteredTextField:setText(ptext)
	self.mytextfield:setText(ptext)
	self.mytextfield:setAnchorPoint(self.anchorpointx, self.anchorpointy)
end

function xCenteredTextField:setTextColor(pcolor)
	self.mytextfield:setTextColor(pcolor)
end

-- ******************************************************************************

-- ******************************************************************************

xCenteredTextFieldW = Core.class(Sprite)

function xCenteredTextFieldW:init(ptext, pfont, pcolor, pw, ph, pline)
	self.anchorpointx = 0.5
--	self.anchorpointy = 0.5
	self.anchorpointy = 0
	self.mytextfield = TextField.new(pfont, ptext, ptext)
	self.mytextfield:setSample(ptext)

--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_BOTTOM} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_BOTTOM} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_BREAKWORDS} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_CENTER} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_CENTER} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_JUSTIFIED} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_LEFT} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_LEFT} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_NOWRAP} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_REF_BASELINE} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_REF_BASELINE} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_REF_BOTTOM} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_REF_BOTTOM} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_REF_MIDDLE} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_REF_MIDDLE} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_REF_TOP} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_REF_TOP} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_RIGHT} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_TOP} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_TOP} )
--	self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = 7, flags=FontBase.TLF_VCENTER} )
--	self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_VCENTER} )

	if pline == 1 then -- arab center
		self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = -32, flags=FontBase.TLF_CENTER} )
	elseif pline == 2 then -- tr center
		self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_CENTER} )
	elseif pline == 3 then -- tr left
		self.mytextfield:setLayout( {w = pw, h = ph, lineSpacing = -8, flags=FontBase.TLF_LEFT} )
	elseif pline == 4 then -- tr justified
		self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_JUSTIFIED} )
	else -- grammar
		self.mytextfield:setLayout( {w = pw, h = ph, flags=FontBase.TLF_REF_BASELINE} )
	end
	self.mytextfield:setTextColor(pcolor or 0xff0000)
	self.mytextfield:setAnchorPoint(self.anchorpointx, self.anchorpointy)
	self:addChild(self.mytextfield)
end

function xCenteredTextFieldW:setText(ptext)
	self.mytextfield:setText(ptext)
	self.mytextfield:setSample(ptext)
	self.mytextfield:setAnchorPoint(self.anchorpointx, self.anchorpointy)
end

function xCenteredTextFieldW:setFontSize(xfsize)
	self.mytextfield:setFont(xfsize)
end

function xCenteredTextFieldW:setTextColor(xfcolor)
	self.mytextfield:setTextColor(xfcolor)
end

-- ******************************************************************************

-- ******************************************************************************

function xBackgroundO(xtexture, xscale, xalpha)
	local mysprite = Sprite.new()

	local mybgtexture = Texture.new(xtexture)
	local mybgbmp = Bitmap.new(mybgtexture)
	mybgbmp:setScale(xscale)
	mybgbmp:setAlpha(1.0)
	mybgbmp:setAnchorPoint(0.5, 0.5)
	mybgbmp:setPosition(myappwidth / 2, myappheight / 2)
	
	local myoverlay = Shape.new()
	myoverlay:setFillStyle(Shape.SOLID, 0x000000)
	myoverlay:beginPath()
	myoverlay:moveTo(0, 0)
	myoverlay:lineTo(myappwidth * 2, 0)
	myoverlay:lineTo(myappwidth * 2, myappheight * 2)
	myoverlay:lineTo(0, myappheight * 2)
	myoverlay:lineTo(0, 0)
	myoverlay:endPath()
	myoverlay:setAlpha(xalpha)
	myoverlay:setAnchorPoint(0.5, 0.5)
	myoverlay:setPosition(myappwidth / 2, myappheight / 2)

	mysprite:addChild(mybgbmp)
	mysprite:addChild(myoverlay)
	
	return mysprite
end

-- ******************************************************************************

-- ******************************************************************************

function xsaveJPGtoDoc(xoriginalimgfile, xposx, xposy, xsizew, xsizeh, xdestimgname)
	local mytexture = Texture.new(xoriginalimgfile)
	local mybmp = Bitmap.new(mytexture)
	local rt = RenderTarget.new(mybmp:getWidth(), mybmp:getHeight())
	rt:draw(mybmp)
	rt:save("|D|tex_"..xdestimgname..".jpg", xposx, xposy, xsizew, xsizeh)
	
	return "|D|tex_"..xdestimgname..".jpg"
end


