Player = Core.class(Sprite)

function Player:init(xworld, xspritesheet, xcols, xrows)
	-- settings
	self.lives = 1
	self.speed = 3
	self.flip = 1
	self.numfeetcontacts = 0 -- is on ground
	self.remainingjumpsteps = 0 -- jump (apply force) ***
	self.isleft, self.isright, self.isup, self.isdown = false, false, false, false -- keys

	-- animations
	self.currentanim = ""
	self.frame = 0
	self.animspeed = 1 / 6
	self.animtimer = self.animspeed

	-- retrieve all anims in spritesheet
	local myanimstex = Texture.new(xspritesheet)
	local w, h = myanimstex:getWidth() / xcols, myanimstex:getHeight() / xrows
	local myanims_list = {}
	for r = 1, xrows do
		for c = 1, xcols do
			local myanimstexregion = TextureRegion.new(myanimstex, (c - 1) * w, (r - 1) * h, w, h)
			myanims_list[#myanims_list + 1] = myanimstexregion
		end
	end

	-- create anims
	self.anims = {}
	function createAnim(xanimname, xstart, xfinish, xanimslist)
		self.anims[xanimname] = {}
		for i = xstart, xfinish do
			self.anims[xanimname][#self.anims[xanimname] + 1] = xanimslist[i]
		end
	end
	createAnim("idle", 1, 2, myanims_list)
	createAnim("jump_up", 37, 38, myanims_list)
	createAnim("jump_down", 39, 40, myanims_list)
	createAnim("walk", 13, 16, myanims_list)

	-- the sprite & the bitmap
	local mysprite = Sprite.new()
	self.bitmap = Bitmap.new(myanims_list[1]) -- starting bmp texture
	self.bitmap:setAnchorPoint(0.5, 0.5)
	self.bitmap:setPosition(w / 2, h / 2)
	mysprite:addChild(self.bitmap)
	self:addChild(mysprite)

	-- the dynamic body (liquidfun)
	self.body = xworld:createBody { type = b2.DYNAMIC_BODY }
	self.body:setFixedRotation(true)
	local bodyshape = b2.PolygonShape.new()
	bodyshape:setAsBox(w / 4, h / 2, w / 2, h / 2, 0)
	local bodyfixture = self.body:createFixture { shape = bodyshape, density = 1, restitution = 0, friction = 1 }

	-- the left and right body with no friction so does not stick on walls!
	local myleftshape = b2.PolygonShape.new()
	myleftshape:setAsBox(4, h / 4, w / 4, h / 1.5, 0)
	myleftfixture = self.body:createFixture { shape = myleftshape, friction = 0 }
	local myrightshape = b2.PolygonShape.new()
	myrightshape:setAsBox(4, h / 4, w - w / 4, h / 1.5, 0)
	myrightfixture = self.body:createFixture { shape = myrightshape, friction = 0 }

	-- the feet sensor
	local myfeetshape = b2.PolygonShape.new()
	myfeetshape:setAsBox(w / 4, 8, w / 2, h, 0)
	myfeetfixture = self.body:createFixture { shape = myfeetshape }
	myfeetfixture:setSensor(true)
	myfeetfixture.type = 2

	-- keys listeners
	self:addEventListener(Event.KEY_DOWN, self.onKeyDown, self)
	self:addEventListener(Event.KEY_UP, self.onKeyUp, self)
end

-- keys listeners
function Player:onKeyDown(e)
	if e.keyCode == KeyCode.LEFT then self.isleft = true end
	if e.keyCode == KeyCode.RIGHT then self.isright = true end
	if e.keyCode == KeyCode.UP then
		self.isup = true -- JUMP
		if self.numfeetcontacts > 0 then -- is on ground
--[[
			-- 1st method velocity
			local vx, vy = self.body:getLinearVelocity()
			self.body:setLinearVelocity(vx, -1.5 * self.speed)
]]
--[[
			-- 2nd method force ***
			self.remainingjumpsteps = 6
]]
			-- 3rd method impulse
			local impulse = self.body:getMass() * 7.5
			self.body:applyLinearImpulse(0, -impulse, self.body:getWorldCenter())
		end
	end
	if e.keyCode == KeyCode.DOWN then
		self.isdown = true
	end
end

function Player:onKeyUp(e)
	if e.keyCode == KeyCode.RIGHT then self.isright = false end
	if e.keyCode == KeyCode.LEFT then self.isleft = false end
	if e.keyCode == KeyCode.UP then self.isup = false end
	if e.keyCode == KeyCode.DOWN then self.isdown = false end
end
